==> riscv_config-3.5.2.tar.gz: riscv_config-3.5.2/riscv_config/isa_validator.py <==
import riscv_config.constants as constants
from collections import Counter
import re

def get_extension_list(isa):
    extension_list = []
    err = False
    err_list = []
    if not constants.isa_regex.match(isa):
        err = True
        err_list.append('Input ISA string does not match accepted canonical ordering')
        return (extension_list, err, err_list)

    
    str_match = re.findall('(?P<stdisa>[^\d]*?)(?!_)*(?P<zext>Z.*?)*(?P<sext>S[a-z]*)*(?P<xext>X[a-z0-9]*)*(_|$)',isa)
    extension_list= []
    standard_isa = ''
    zext_list = []
    for match in str_match:
        stdisa, zext, sext, xext, ignore = match
        if stdisa != '':
            for e in stdisa:
                extension_list.append(e)
            standard_isa = stdisa
        if zext != '':
            extension_list.append(zext)
            zext_list.append(zext)
        if sext != '':
            extension_list.append(sext)
        if xext != '':
            extension_list.append(xext)
    # check for duplicates
    counts = Counter(extension_list)
    duplicate_list = list([item for item in counts if counts[item]>1])
    if duplicate_list:
        err = True
        err_list.append(f'Found duplicate extensions in ISA string: {duplicate_list}')
        return (extension_list, err, err_list)

    # check ordering of ISA
    canonical_ordering = 'IEMAFDQLCBJKTPVNSHU'
    order_index = {c: i for i, c in enumerate(canonical_ordering)}
    for i in range(len(standard_isa)-1):
        a1 = standard_isa[i]
        a2 = standard_isa[i+1]
    
        if order_index[a1] > order_index[a2]:
            err = True
            err_list.append( "Alphabet '" + a1 + "' should occur after '" + a2)

    # check canonical ordering within Zextensions
    for i in range(len(zext_list)-1):
        a1 = zext_list[i][1].upper()
        a2 = zext_list[i+1][1].upper()
        a3 = zext_list[i][2]
        a4 = zext_list[i+1][2]
        if order_index[a1] > order_index[a2]:
            err = True
            err_list.append( f"Z extension {zext_list[i]} must occur after {zext_list[i+1]}")
        elif a1 == a2 and a3 > a4:
            err = True
            err_list.append( f"Within the Z{a1.lower()} category extension {zext_list[i]} must occur after {zext_list[i+1]}")
        
    if 'I' not in extension_list and 'E' not in extension_list:
        err_list.append( 'Either of I or E base extensions need to be present in the ISA string')
        err = True
    if 'F' in extension_list and not "Zicsr" in extension_list:
        err_list.append( "F cannot exist without Zicsr.")
        err = True
    if 'D' in extension_list and not 'F' in extension_list:
        err_list.append( "D cannot exist without F.")
        err = True
    if 'Q' in extension_list and not 'D' in extension_list:
        err_list.append( "Q cannot exist without D and F.")
        err = True
    if 'Zam' in extension_list and not 'A' in extension_list:
        err_list.append( "Zam cannot exist without A.")
        err = True
    if 'N' in extension_list and not 'U' in extension_list:
        err_list.append( "N cannot exist without U.")
        err = True
    if 'S' in extension_list and not 'U' in extension_list:
        err_list.append( "S cannot exist without U.")
        err = True
    if 'Zkn' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zkne', 'Zknd', 'Zknh']) & set(extension_list)):
        err_list.append( "Zkn is a superset of Zbkb, Zbkc, Zbkx, Zkne, Zknd, Zknh. In presence of Zkn the subsets must be ignored in the ISA string.")
        err = True
    if 'Zks' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zksed', 'Zksh']) & set(extension_list) ):
        err_list.append( "Zks is a superset of Zbkb, Zbkc, Zbkx, Zksed, Zksh. In presence of Zks the subsets must be ignored in the ISA string.")
        err = True
    if 'Zk' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zkne', 'Zknd',
        'Zknh', 'Zkn','Zkr','Zkt']) & set(extension_list)):
        err_list.append( "Zkn is a superset of Zbkb, Zbkc, Zbkx, Zkne, Zknd, Zknh, Zkn, Zkt, Zkr. In presence of Zkn the subsets must be ignored in the ISA string.")
        err = True
    if 'Zfinx' in extension_list and not "Zicsr" in extension_list:
        err_list.append( "Zfinx cannot exist without Zicsr.")
        err = True
    if 'F' in extension_list and "Zfinx" in extension_list:
        err_list.append( "F and Zfinx cannot exist together")
        err = True
    if 'Zdinx' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zdinx cannot exist without Zfinx.")
        err = True
    if 'Zhinx' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zhinx cannot exist without Zhinx.")
        err = True
    if 'Zhinx' in extension_list and 'Zfh' in extension_list:
        err_list.append( "Zhinx and Zfh cannot exist together.")
        err = True
    if 'Zhinxmin' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zhinxmin cannot exist without Zhinx.")
        err = True
    if 'Zhinxmin' in extension_list and 'Zfh' in extension_list:
        err_list.append( "Zhinxmin and Zfh cannot exist together.")
        err = True
    if 'Zbpbo' in extension_list :
        if not 'Zpn' in extension_list :
            err_list.append( "'Zpn' is required.")
            err = True
    if 'Zpn' in extension_list :
        if not 'Zbpbo' in extension_list :
            err_list.append( "'Zbpbo' is required.")
            err = True
        if '64' in isa and not 'Zpsf' in extension_list :
            err_list.append( "RV64 requires 'Zpsf'.")
            err = True
        if not ('M' in extension_list or 'Zmmul' in extension_list):
            err_list.append("baseline multiplication extension not found.")
    if 'Zpsf' in extension_list and not err :
        if not 'Zbpbo' in extension_list :
            err_list.append( "'Zbpbo' is required.")
            err = True
        if not 'Zpn' in extension_list :
            err_list.append( "'Zpn' is required.")
            err = True

    return (extension_list, err, err_list)

==> riscv_config-3.14.1.tar.gz: riscv_config-3.14.1/riscv_config/isa_validator.py <==
import riscv_config.constants as constants
from collections import Counter
import re

def get_extension_list(isa):
    extension_list = []
    err = False
    err_list = []
    if not constants.isa_regex.match(isa):
        err = True
        err_list.append(f'Input ISA string : {isa} does not match accepted canonical ordering')
        return (extension_list, err, err_list)

    
    str_match = re.findall('(?P<stdisa>[^\d]*?)(?!_)*(?P<zext>Z.*?)*(?P<sext>S[a-z]*)*(?P<xext>X[a-z0-9]*)*(_|$)',isa)
    extension_list= []
    standard_isa = ''
    zext_list = []
    for match in str_match:
        stdisa, zext, sext, xext, ignore = match
        if stdisa != '':
            for e in stdisa:
                extension_list.append(e)
            standard_isa = stdisa
        if zext != '':
            extension_list.append(zext)
            zext_list.append(zext)
        if sext != '':
            extension_list.append(sext)
        if xext != '':
            extension_list.append(xext)
    # check for duplicates
    counts = Counter(extension_list)
    duplicate_list = list([item for item in counts if counts[item]>1])
    if duplicate_list:
        err = True
        err_list.append(f'Found duplicate extensions in ISA string: {duplicate_list}')
        return (extension_list, err, err_list)

    # check ordering of ISA
    canonical_ordering = 'IEMAFDQLCBJKTPVNSHU'
    order_index = {c: i for i, c in enumerate(canonical_ordering)}
    for i in range(len(standard_isa)-1):
        a1 = standard_isa[i]
        a2 = standard_isa[i+1]
    
        if order_index[a1] > order_index[a2]:
            err = True
            err_list.append( "Alphabet '" + a1 + "' should occur after '" + a2)

    # check canonical ordering within Zextensions
    for i in range(len(zext_list)-1):
        a1 = zext_list[i][1].upper()
        a2 = zext_list[i+1][1].upper()
        a3 = zext_list[i][2]
        a4 = zext_list[i+1][2]
        if order_index[a1] > order_index[a2]:
            err = True
            err_list.append( f"Z extension {zext_list[i]} must occur after {zext_list[i+1]}")
        elif a1 == a2 and a3 > a4:
            err = True
            err_list.append( f"Within the Z{a1.lower()} category extension {zext_list[i]} must occur after {zext_list[i+1]}")
        
    if 'I' not in extension_list and 'E' not in extension_list:
        err_list.append( 'Either of I or E base extensions need to be present in the ISA string')
        err = True
    if 'F' in extension_list and not "Zicsr" in extension_list:
        err_list.append( "F cannot exist without Zicsr.")
        err = True
    if 'D' in extension_list and not 'F' in extension_list:
        err_list.append( "D cannot exist without F.")
        err = True
    if 'Q' in extension_list and not 'D' in extension_list:
        err_list.append( "Q cannot exist without D and F.")
        err = True
    if 'Zam' in extension_list and not 'A' in extension_list:
        err_list.append( "Zam cannot exist without A.")
        err = True
    if 'N' in extension_list and not 'U' in extension_list:
        err_list.append( "N cannot exist without U.")
        err = True
    if 'S' in extension_list and not 'U' in extension_list:
        err_list.append( "S cannot exist without U.")
        err = True
    if 'Zkn' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zkne', 'Zknd', 'Zknh']) & set(extension_list)):
        err_list.append( "Zkn is a superset of Zbkb, Zbkc, Zbkx, Zkne, Zknd, Zknh. In presence of Zkn the subsets must be ignored in the ISA string.")
        err = True
    if 'Zks' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zksed', 'Zksh']) & set(extension_list) ):
        err_list.append( "Zks is a superset of Zbkb, Zbkc, Zbkx, Zksed, Zksh. In presence of Zks the subsets must be ignored in the ISA string.")
        err = True
    if 'Zk' in extension_list and ( set(['Zbkb', 'Zbkc', 'Zbkx', 'Zkne', 'Zknd',
        'Zknh', 'Zkn','Zkr','Zkt']) & set(extension_list)):
        err_list.append( "Zk is a superset of Zbkb, Zbkc, Zbkx, Zkne, Zknd, Zknh, Zkn, Zkt, Zkr. In presence of Zk the subsets must be ignored in the ISA string.")
        err = True
    if 'Zfinx' in extension_list and not "Zicsr" in extension_list:
        err_list.append( "Zfinx cannot exist without Zicsr.")
        err = True
    if 'F' in extension_list and "Zfinx" in extension_list:
        err_list.append( "F and Zfinx cannot exist together")
        err = True
    if 'Zdinx' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zdinx cannot exist without Zfinx.")
        err = True
    if 'Zhinx' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zhinx cannot exist without Zfinx.")
        err = True
    if 'Zhinx' in extension_list and 'Zfh' in extension_list:
        err_list.append( "Zhinx and Zfh cannot exist together.")
        err = True
    if 'Zhinxmin' in extension_list and not 'Zfinx' in extension_list:
        err_list.append( "Zhinxmin cannot exist without Zfinx.")
        err = True
    if 'Zhinxmin' in extension_list and 'Zfh' in extension_list:
        err_list.append( "Zhinxmin and Zfh cannot exist together.")
        err = True
    if 'Zfa' in extension_list and not 'Zfh' in extension_list:
        err_list.append( "Zfa cannot exist without Zfh.")
    if 'Zbpbo' in extension_list :
        if not 'Zpn' in extension_list :
            err_list.append( "'Zpn' is required.")
            err = True
    if 'Zpn' in extension_list :
        if not 'Zbpbo' in extension_list :
            err_list.append( "'Zbpbo' is required.")
            err = True
        if '64' in isa and not 'Zpsf' in extension_list :
            err_list.append( "RV64 requires 'Zpsf'.")
            err = True
        if not ('M' in extension_list or 'Zmmul' in extension_list):
            err_list.append("baseline multiplication extension not found.")
    if 'Zpsf' in extension_list and not err :
        if not 'Zbpbo' in extension_list :
            err_list.append( "'Zbpbo' is required.")
            err = True
        if not 'Zpn' in extension_list :
            err_list.append( "'Zpn' is required.")
            err = True

    # Check V extensions
    if 'V' in extension_list and 'D' not in extension_list:
        err_list.append("V cannot exist without D and F.")
        err = True
    if len(set(['V'] + constants.Zve_extensions) & set(extension_list)) > 1:
        err_list.append(f"V and Zve* cannot exist together.")
        err = True
    if (set(constants.Zvl_extensions) & set(extension_list)) and not (
        set(['V'] + constants.Zve_extensions) & set(extension_list)
    ):
        err_list.append("Zvl* cannot exist without V or Zve*.")
        err = True
    if set(constants.Zvef_extensions) & set(extension_list) and (
            {"F", "Zfinx"} & set(extension_list)
    ):
        err_list.append("Zve*f cannot exist without F or Zfinx.")
        err = True
    if (set(constants.Zved_extensions) & set(extension_list)) and not (
            {"D", "Zdinx"} & set(extension_list)
    ):
        err_list.append("Zve64d cannot exist without D or Zdinx.")
        err = True

    return (extension_list, err, err_list)
